@@ -4,7 +4,6 @@ import boto3
 import json
 import time
 import copy
-import logging
 import importlib
 import shutil
 import subprocess
@@ -13,6 +12,7 @@ from datetime import datetime, timedelta
 from dateutil.tz import tzutc
 from uuid import uuid4, UUID
 from types import ModuleType
+from . import create_logger
 from .vars import (
     _tibanna,
     AWS_ACCOUNT_NUMBER,
@@ -20,11 +20,7 @@ from .vars import (
     TIBANNA_DEFAULT_STEP_FUNCTION_NAME,
     STEP_FUNCTION_ARN,
     EXECUTION_ARN,
-    AMI_ID_CWL_V1,
-    AMI_ID_CWL_DRAFT3,
-    AMI_ID_WDL,
-    AMI_ID_SNAKEMAKE,
-    AMI_ID_SHELL,
+    AMI_ID,
     TIBANNA_REPO_NAME,
     TIBANNA_REPO_BRANCH,
     TIBANNA_PROFILE_ACCESS_KEY,
@@ -39,7 +35,6 @@ from .vars import (
 )
 from .utils import (
     _tibanna_settings,
-    printlog,
     create_jobid,
     does_key_exist,
     read_s3,
@@ -51,6 +46,7 @@ from .ec2_utils import (
     UnicornInput,
     upload_workflow_to_s3
 )
+from .ami import AMI
 # from botocore.errorfactory import ExecutionAlreadyExists
 from .stepfunction import StepFunctionUnicorn
 from .awsem import AwsemRunJson, AwsemPostRunJson
@@ -61,8 +57,7 @@ from . import dd_utils
 
 
 # logger
-LOG = logging.getLogger(__name__)
-LOG.setLevel(logging.INFO)
+logger = create_logger(__name__)
 
 
 UNICORN_LAMBDAS = ['run_task_awsem', 'check_task_awsem']
@@ -183,14 +178,14 @@ class API(object):
             unicorn_input = UnicornInput(data)
             args = unicorn_input.args
             if args.language.startswith('cwl') and args.cwl_directory_local or \
-               args.language == 'wdl' and args.wdl_directory_local or \
+               args.language in ['wdl', 'wdl_v1', 'wdl_draft2'] and args.wdl_directory_local or \
                args.language == 'snakemake' and args.snakemake_directory_local:
                 upload_workflow_to_s3(unicorn_input)
                 data['args'] = args.as_dict()  # update args
         # submit job as an execution
         aws_input = json.dumps(data)
         if verbose:
-            print("about to start run %s" % run_name)
+            logger.info("about to start run %s" % run_name)
         # trigger the step function to run
         try:
             response = client.start_execution(
@@ -206,19 +201,29 @@ class API(object):
         data[_tibanna]['response'] = response
         if verbose:
             # print some info
-            print("response from aws was: \n %s" % response)
-            print("url to view status:")
-            print(data[_tibanna]['url'])
-            print("JOBID %s submitted" % data['jobid'])
-            print("EXECUTION ARN = %s" % data[_tibanna]['exec_arn'])
+            logger.info("response from aws was: \n %s" % response)
+            logger.info("url to view status:")
+            logger.info(data[_tibanna]['url'])
+            logger.info("JOBID %s submitted" % data['jobid'])
+            logger.info("EXECUTION ARN = %s" % data[_tibanna]['exec_arn'])
             if 'cloudwatch_dashboard' in data['config'] and data['config']['cloudwatch_dashboard']:
                 cw_db_url = 'https://console.aws.amazon.com/cloudwatch/' + \
                     'home?region=%s#dashboards:name=awsem-%s' % (AWS_REGION, jobid)
-                print("Cloudwatch Dashboard = %s" % cw_db_url)
+                logger.info("Cloudwatch Dashboard = %s" % cw_db_url)
             if open_browser and shutil.which('open') is not None:
                 subprocess.call(["open", data[_tibanna]['url']])
         return data
 
+    def run_batch_workflows(self, input_json_list, sfn=None,
+                     env=None, sleep=3, verbose=True, open_browser=True):
+        """given a list of input json, run multiple workflows"""
+        run_infos = []
+        for input_json in input_json_list:
+            run_info = self.run_workflow(input_json, env=env, sfn=sfn, sleep=sleep, verbose=verbose,
+                       open_browser=False)
+            run_infos.append(run_info)
+        return run_infos
+
     def add_to_dydb(self, awsem_job_id, execution_name, sfn, logbucket, verbose=True):
         time_stamp = datetime.strftime(datetime.utcnow(), '%Y%m%d-%H:%M:%S-UTC')
         dydb = boto3.client('dynamodb', region_name=AWS_REGION)
@@ -227,7 +232,7 @@ class API(object):
             res = dydb.describe_table(TableName=DYNAMODB_TABLE)
         except Exception as e:
             if verbose:
-                printlog("Not adding to dynamo table: %s" % e)
+                logger.info("Not adding to dynamo table: %s" % e)
             return
         try:
             response = dydb.put_item(
@@ -251,7 +256,7 @@ class API(object):
                 }
             )
             if verbose:
-                printlog(response)
+                logger.info("Successfully put item to dynamoDB: " + str(response))
         except Exception as e:
             raise(e)
 
@@ -262,7 +267,9 @@ class API(object):
             ddinfo = self.info(job_id)
             if not ddinfo:
                 raise Exception("Can't find exec_arn from the job_id")
-            exec_arn = ddinfo.get('exec_arn', '')
+            exec_name = ddinfo.get('Execution Name', '')
+            sfn = ddinfo.get('Step Function', '')
+            exec_arn = EXECUTION_ARN(exec_name, sfn)
             if not exec_arn:
                 raise Exception("Can't find exec_arn from the job_id")
         sts = boto3.client('stepfunctions', region_name=AWS_REGION)
@@ -275,7 +282,9 @@ class API(object):
             ddinfo = self.info(job_id)
             if not ddinfo:
                 raise Exception("Can't find exec_arn from the job_id")
-            exec_arn = ddinfo.get('exec_arn', '')
+            exec_name = ddinfo.get('Execution Name', '')
+            sfn = ddinfo.get('Step Function', '')
+            exec_arn = EXECUTION_ARN(exec_name, sfn)
             if not exec_arn:
                 raise Exception("Can't find exec_arn from the job_id")
         sts = boto3.client('stepfunctions', region_name=AWS_REGION)
@@ -296,7 +305,7 @@ class API(object):
                                                        'ComparisonOperator': 'EQ'}})
             return ddres
         except Exception as e:
-            printlog("Warning: dynamoDB entry not found: %s" % e)
+            logger.warning("DynamoDB entry not found: %s" % e)
             return None
 
     def info(self, job_id):
@@ -311,27 +320,12 @@ class API(object):
         if 'Items' in ddres:
             try:
                 dditem = ddres['Items'][0]
-                exec_name = dditem['Execution Name']['S']
-                sfn = dditem['Step Function']['S']
-                exec_arn = EXECUTION_ARN(exec_name, sfn)
-                if 'instance_id' in dditem:
-                    instance_id = dditem['instance_id']['S']
-                else:
-                    instance_id = ''
-                if 'Log Bucket' in dditem:
-                    logbucket = dditem['Log Bucket']['S']
-                else:
-                    logbucket = ''
-                return {'exec_name': exec_name,
-                        'exec_arn': exec_arn,
-                        'step_function': sfn,
-                        'instance_id': instance_id,
-                        'log_bucket': logbucket}
+                return dd_utils.item2dict(dditem)
             except Exception as e:
-                printlog("Warning: dynamoDB fields not found: %s" % e)
+                logger.warning("DynamoDB fields not found: %s" % e)
                 return None
         else:
-            printlog("Warning: dynamoDB Items field not found:")
+            logger.warning("DynamoDB Items field not found:")
             return None
 
     def kill(self, exec_arn=None, job_id=None, sfn=None):
@@ -348,25 +342,25 @@ class API(object):
                             if tag['Key'] == 'Type' and tag['Value'] != 'awsem':
                                 continue
                             if tag['Key'] == 'Name' and tag['Value'] == 'awsem-' + jobid:
-                                printlog("terminating EC2 instance")
+                                logger.info("terminating EC2 instance")
                                 response = i.terminate()
-                                printlog(response)
+                                logger.info("Successfully terminated instance: " + str(response))
                                 terminated = True
                                 break
                         if terminated:
                             break
-                printlog("terminating step function execution")
+                logger.info("terminating step function execution")
                 resp_sf = sf.stop_execution(executionArn=exec_arn, error="Aborted")
-                printlog(resp_sf)
+                logger.info("Successfully terminated step function execution: " + str(resp_sf))
         elif job_id:
             ec2 = boto3.client('ec2')
             res = ec2.describe_instances(Filters=[{'Name': 'tag:Name', 'Values': ['awsem-' + job_id]}])
             if not res['Reservations']:
                 raise("instance not available - if you just submitted the job, try again later")
             instance_id = res['Reservations'][0]['Instances'][0]['InstanceId']
-            printlog("terminating EC2 instance")
+            logger.info("terminating EC2 instance")
             resp_term = ec2.terminate_instances(InstanceIds=[instance_id])
-            printlog(resp_term)
+            logger.info("Successfully terminated instance: " + str(resp_term))
             # first try dynanmodb to get logbucket
             ddres = dict()
             try:
@@ -382,7 +376,7 @@ class API(object):
                 exec_arn = EXECUTION_ARN(exec_name, sfn)
             else:
                 if not sfn:
-                    printlog("Can't stop step function because step function name is not given.")
+                    logger.warning("Can't stop step function because step function name is not given.")
                     return None
                 stateMachineArn = STEP_FUNCTION_ARN(sfn)
                 res = sf.list_executions(stateMachineArn=stateMachineArn, statusFilter='RUNNING')
@@ -404,9 +398,9 @@ class API(object):
                         break
                 if not exec_arn:
                     raise Exception("can't find the execution")
-            printlog("terminating step function execution")
+            logger.info("terminating step function execution")
             resp_sf = sf.stop_execution(executionArn=exec_arn, error="Aborted")
-            printlog(resp_sf)
+            logger.info("Successfully terminated step function execution: " + str(resp_sf))
 
     def kill_all(self, sfn=None):
         """killing all the running jobs"""
@@ -426,11 +420,19 @@ class API(object):
             else:
                 break
 
-    def log(self, exec_arn=None, job_id=None, exec_name=None, sfn=None, postrunjson=False, runjson=False, logbucket=None, quiet=False):
+    def log(self, exec_arn=None, job_id=None, exec_name=None, sfn=None,
+            postrunjson=False, runjson=False, top=False, top_latest=False,
+            inputjson=False, logbucket=None, quiet=False):
         if postrunjson:
             suffix = '.postrun.json'
         elif runjson:
             suffix = '.run.json'
+        elif top:
+            suffix = '.top'
+        elif top_latest:
+            suffix = '.top_latest'
+        elif inputjson:
+            suffix = '.input.json'
         else:
             suffix = '.log'
         if not sfn:
@@ -446,17 +448,11 @@ class API(object):
         elif job_id:
             if not logbucket:
                 # first try dynanmodb to get logbucket
-                ddres = dict()
                 try:
-                    dd = boto3.client('dynamodb')
-                    ddres = dd.query(TableName=DYNAMODB_TABLE,
-                                     KeyConditions={'Job Id': {'AttributeValueList': [{'S': job_id}],
-                                                               'ComparisonOperator': 'EQ'}})
+                    logbucket = self.info(job_id)['Log Bucket']
                 except Exception as e:
                     pass
-                if 'Items' in ddres:
-                    logbucket = ddres['Items'][0]['Log Bucket']['S']
-                else:
+                if not logbucket:
                     # search through executions to get logbucket
                     stateMachineArn = STEP_FUNCTION_ARN(sfn)
                     try:
@@ -487,8 +483,8 @@ class API(object):
         except Exception as e:
             if 'NoSuchKey' in str(e):
                 if not quiet:
-                    printlog("log/postrunjson file is not ready yet. " +
-                             "Wait a few seconds/minutes and try again.")
+                    logger.info("log/postrunjson file is not ready yet. " +
+                                "Wait a few seconds/minutes and try again.")
                 return ''
             else:
                 raise e
@@ -496,20 +492,17 @@ class API(object):
             return(res_s3['Body'].read().decode('utf-8', 'backslashreplace'))
         return None
 
-    def stat(self, sfn=None, status=None, verbose=False, n=None):
+    def stat(self, sfn=None, status=None, verbose=False, n=None, job_ids=None):
         """print out executions with details (-v)
         status can be one of 'RUNNING'|'SUCCEEDED'|'FAILED'|'TIMED_OUT'|'ABORTED'
+        or specify a list of job ids
         """
-        if not sfn:
-            sfn = self.default_stepfunction_name
-        args = {
-            'stateMachineArn': STEP_FUNCTION_ARN(sfn),
-            'maxResults': 100
-        }
-        if status:
-            args['statusFilter'] = status
-        res = dict()
-        client = boto3.client('stepfunctions')
+        if n and job_ids:
+            raise Exception("n and job_id filters do not work together.")
+        if sfn and job_ids:
+            raise Exception("Please do not specify sfn when job_ids are specified.")
+        if status and job_ids:
+            raise Exception("Status filter cannot be specified when job_ids are specified.")
         if verbose:
             print("{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}".format('jobid', 'status', 'name',
                                                                       'start_time', 'stop_time',
@@ -518,59 +511,92 @@ class API(object):
                                                                       'password'))
         else:
             print("{}\t{}\t{}\t{}\t{}".format('jobid', 'status', 'name', 'start_time', 'stop_time'))
-        res = client.list_executions(**args)
+        client = boto3.client('stepfunctions')
         ec2 = boto3.client('ec2')
-        k = 0
-        while True:
-            if n and k == n:
-                break
-            if 'executions' not in res or not res['executions']:
-                break
-            for exc in res['executions']:
+
+        def parse_exec_desc_and_ec2_desc(exec_arn, verbose):
+            # collecting execution stats
+            exec_desc = client.describe_execution(executionArn=exec_arn)
+
+            # getting info from execution description
+            exec_desc = client.describe_execution(executionArn=exec_arn)
+            job_id = json.loads(exec_desc['input']).get('jobid', 'no jobid')
+            status = exec_desc['status']
+            name = exec_desc['name']
+            start_time = exec_desc['startDate'].strftime("%Y-%m-%d %H:%M")
+            if 'stopDate' in exec_desc:
+                stop_time = exec_desc['stopDate'].strftime("%Y-%m-%d %H:%M")
+            else:
+                stop_time = ''
+
+            # collect instance stats
+            ec2_desc = ec2.describe_instances(Filters=[{'Name': 'tag:Name', 'Values': ['awsem-' + job_id]}])
+
+            # getting info from ec2 description
+            if ec2_desc['Reservations']:
+                ec2_desc_inst = ec2_desc['Reservations'][0]['Instances'][0]
+                instance_status = ec2_desc_inst['State']['Name']
+                instance_id = ec2_desc_inst['InstanceId']
+                instance_type = ec2_desc_inst['InstanceType']
+                if instance_status not in ['terminated', 'shutting-down']:
+                    instance_ip = ec2_desc_inst.get('PublicIpAddress', '-')
+                    keyname = ec2_desc_inst.get('KeyName', '-')
+                    password = json.loads(exec_desc['input'])['config'].get('password', '-')
+                else:
+                    instance_ip = '-'
+                    keyname = '-'
+                    password = '-'
+            else:
+                instance_status = '-'
+                instance_id = '-'
+                instance_type = '-'
+                instance_ip = '-'
+                keyname = '-'
+                password = '-'
+
+            parsed_stat = (job_id, status, name, start_time, stop_time,
+                           instance_id, instance_type, instance_status,
+                           instance_ip, keyname, password)
+            if verbose:
+                print("{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}".format(*parsed_stat))
+            else:
+                print("{}\t{}\t{}\t{}\t{}".format(*parsed_stat[0:5]))
+
+        if job_ids:
+            for job_id in job_ids:
+                dd_info = self.info(job_id)
+                if 'Execution Name' not in dd_info:
+                    raise Exception("Cannot find execution name for job ID %s" % job_id)
+                if 'Step Function' not in dd_info:
+                    raise Exception("Cannot find step function for job ID %s" % job_id)
+                exec_arn = EXECUTION_ARN(dd_info['Execution Name'], dd_info['Step Function'])
+                parse_exec_desc_and_ec2_desc(exec_arn, verbose)
+        else:
+            if not sfn:
+                sfn = self.default_stepfunction_name
+            args = {
+                'stateMachineArn': STEP_FUNCTION_ARN(sfn),
+                'maxResults': 100
+            }
+            if status:
+                args['statusFilter'] = status
+            res = dict()
+            res = client.list_executions(**args)
+            k = 0
+            while True:
                 if n and k == n:
                     break
-                k = k + 1
-                desc = client.describe_execution(executionArn=exc['executionArn'])
-                jobid = json.loads(desc['input']).get('jobid', 'no jobid')
-                status = exc['status']
-                name = exc['name']
-                start_time = exc['startDate'].strftime("%Y-%m-%d %H:%M")
-                if 'stopDate' in exc:
-                    stop_time = exc['stopDate'].strftime("%Y-%m-%d %H:%M")
+                if 'executions' not in res or not res['executions']:
+                    break
+                for exc in res['executions']:
+                    if n and k == n:
+                        break
+                    k = k + 1
+                    parse_exec_desc_and_ec2_desc(exc['executionArn'], verbose)
+                if 'nextToken' in res:
+                    res = client.list_executions(nextToken=res['nextToken'], **args)
                 else:
-                    stop_time = ''
-                if verbose:
-                    # collect instance stats
-                    res = ec2.describe_instances(Filters=[{'Name': 'tag:Name', 'Values': ['awsem-' + jobid]}])
-                    if res['Reservations']:
-                        instance_status = res['Reservations'][0]['Instances'][0]['State']['Name']
-                        instance_id = res['Reservations'][0]['Instances'][0]['InstanceId']
-                        instance_type = res['Reservations'][0]['Instances'][0]['InstanceType']
-                        if instance_status not in ['terminated', 'shutting-down']:
-                            instance_ip = res['Reservations'][0]['Instances'][0].get('PublicIpAddress', '-')
-                            keyname = res['Reservations'][0]['Instances'][0].get('KeyName', '-')
-                            password = json.loads(desc['input'])['config'].get('password', '-')
-                        else:
-                            instance_ip = '-'
-                            keyname = '-'
-                            password = '-'
-                    else:
-                        instance_status = '-'
-                        instance_id = '-'
-                        instance_type = '-'
-                        instance_ip = '-'
-                        keyname = '-'
-                        password = '-'
-                    print_template = "{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}"
-                    print(print_template.format(jobid, status, name, start_time, stop_time,
-                                                instance_id, instance_type, instance_status,
-                                                instance_ip, keyname, password))
-                else:
-                    print("{}\t{}\t{}\t{}\t{}".format(jobid, status, name, start_time, stop_time))
-            if 'nextToken' in res:
-                res = client.list_executions(nextToken=res['nextToken'], **args)
-            else:
-                break
+                    break
 
     def list_sfns(self, numbers=False):
         """list all step functions, optionally with a summary (-n)"""
@@ -618,7 +644,7 @@ class API(object):
             input_json_template['_tibanna']['run_name'] = input_json_template['_tibanna']['run_name'][:-36]
             input_json_template['config']['run_name'] = input_json_template['_tibanna']['run_name']
 
-    def rerun(self, exec_arn, sfn=None,
+    def rerun(self, exec_arn=None, job_id=None, sfn=None,
               override_config=None, app_name_filter=None,
               instance_type=None, shutdown_min=None, ebs_size=None, ebs_type=None, ebs_iops=None,
               overwrite_input_extra=None, key_name=None, name=None):
@@ -629,10 +655,13 @@ class API(object):
         then rerun only if it matches app_name
         """
         if not sfn:
-            sfn = self.default_stepfunction_name
-        client = boto3.client('stepfunctions')
-        res = client.describe_execution(executionArn=exec_arn)
-        input_json_template = json.loads(res['input'])
+            sfn = self.default_stepfunction_name  # this is a target sfn
+        if not exec_arn and job_id:
+            input_json_template = json.loads(self.log(job_id=job_id, inputjson=True))
+        else:
+            client = boto3.client('stepfunctions')
+            res = client.describe_execution(executionArn=exec_arn)
+            input_json_template = json.loads(res['input'])
         # filter by app_name
         if app_name_filter:
             if 'app_name' not in input_json_template:
@@ -689,8 +718,6 @@ class API(object):
         sflist = client.list_executions(stateMachineArn=STEP_FUNCTION_ARN(sfn), statusFilter=status)
         k = 0
         for exc in sflist['executions']:
-            print(exc['stopDate'].replace(tzinfo=None))
-            print(stoptime_in_datetime)
             if exc['stopDate'].replace(tzinfo=None) > stoptime_in_datetime:
                 k = k + 1
                 self.rerun(exc['executionArn'], sfn=sfn,
@@ -703,13 +730,9 @@ class API(object):
     def env_list(self, name):
         # don't set this as a global, since not all tasks require it
         envlist = {
-            self.run_task_lambda: {'AMI_ID_CWL_V1': AMI_ID_CWL_V1,
-                               'AMI_ID_CWL_DRAFT3': AMI_ID_CWL_DRAFT3,
-                               'AMI_ID_WDL': AMI_ID_WDL,
-                               'AMI_ID_SNAKEMAKE': AMI_ID_SNAKEMAKE,
-                               'AMI_ID_SHELL': AMI_ID_SHELL,
-                               'TIBANNA_REPO_NAME': TIBANNA_REPO_NAME,
-                               'TIBANNA_REPO_BRANCH': TIBANNA_REPO_BRANCH},
+            self.run_task_lambda: {'AMI_ID': AMI_ID,
+                                   'TIBANNA_REPO_NAME': TIBANNA_REPO_NAME,
+                                   'TIBANNA_REPO_BRANCH': TIBANNA_REPO_BRANCH},
             self.check_task_lambda: {}
         }
         if TIBANNA_PROFILE_ACCESS_KEY and TIBANNA_PROFILE_SECRET_KEY:
@@ -741,14 +764,14 @@ class API(object):
             else:
                 extra_config['Environment']['Variables']['AWS_S3_ROLE_NAME'] = 'S3_access'  # 4dn-dcic default(temp)
         # add role
-        print('name=%s' % name)
+        logger.info('name=%s' % name)
         if name in [self.run_task_lambda, self.check_task_lambda]:
             role_arn_prefix = 'arn:aws:iam::' + AWS_ACCOUNT_NUMBER + ':role/'
             if usergroup:
                 role_arn = role_arn_prefix + tibanna_iam.role_name(name)
             else:
                 role_arn = role_arn_prefix + 'lambda_full_s3'  # 4dn-dcic default(temp)
-            print("role_arn=" + role_arn)
+            logger.info("role_arn=" + role_arn)
             extra_config['Role'] = role_arn
         if usergroup and suffix:
             function_name_suffix = usergroup + '_' + suffix
@@ -767,7 +790,7 @@ class API(object):
         if name not in self.do_not_delete:
             try:
                 boto3.client('lambda').get_function(FunctionName=full_function_name)
-                print("deleting existing lambda")
+                logger.info("deleting existing lambda")
                 boto3.client('lambda').delete_function(FunctionName=full_function_name)
             except Exception as e:
                 if 'Function not found' in str(e):
@@ -780,7 +803,7 @@ class API(object):
 
     def deploy_core(self, name, suffix=None, usergroup=''):
         """deploy/update lambdas only"""
-        print("preparing for deploy...")
+        logger.info("preparing for deploy...")
         if name == 'all':
             names = self.lambda_names
         elif name == 'unicorn':
@@ -795,15 +818,15 @@ class API(object):
         """set up usergroup environment on AWS
         This function is called automatically by deploy_tibanna or deploy_unicorn
         Use it only when the IAM permissions need to be reset"""
-        print("setting up tibanna usergroup environment on AWS...")
+        logger.info("setting up tibanna usergroup environment on AWS...")
         if not AWS_ACCOUNT_NUMBER or not AWS_REGION:
-            print("Please set and export environment variable AWS_ACCOUNT_NUMBER and AWS_REGION!")
+            logger.info("Please set and export environment variable AWS_ACCOUNT_NUMBER and AWS_REGION!")
             exit(1)
         if not buckets:
-            print("WARNING: Without setting buckets (using --buckets)," +
-                  "Tibanna would have access to only public buckets." +
-                  "To give permission to Tibanna for private buckets," +
-                  "use --buckets=<bucket1>,<bucket2>,...")
+            logger.warning("Without setting buckets (using --buckets)," +
+                           "Tibanna would have access to only public buckets." +
+                           "To give permission to Tibanna for private buckets," +
+                           "use --buckets=<bucket1>,<bucket2>,...")
             time.sleep(2)
         if buckets:
             bucket_names = buckets.split(',')
@@ -812,11 +835,11 @@ class API(object):
         if bucket_names and not do_not_delete_public_access_block:
             client = boto3.client('s3')
             for b in bucket_names:
-                printlog("Deleting public access block for bucket %s" % b)
+                logger.info("Deleting public access block for bucket %s" % b)
                 response = client.delete_public_access_block(Bucket=b)
         tibanna_iam = self.IAM(usergroup_tag, bucket_names, no_randomize=no_randomize)
         tibanna_iam.create_tibanna_iam(verbose=verbose)
-        print("Tibanna usergroup %s has been created on AWS." % tibanna_iam.user_group_name)
+        logger.info("Tibanna usergroup %s has been created on AWS." % tibanna_iam.user_group_name)
         return tibanna_iam.user_group_name
 
     def deploy_tibanna(self, suffix=None, usergroup='', setup=False,
@@ -831,12 +854,12 @@ class API(object):
                             do_not_delete_public_access_block=do_not_delete_public_access_block)
         # this function will remove existing step function on a conflict
         step_function_name = self.create_stepfunction(suffix, usergroup=usergroup)
-        print("creating a new step function... %s" % step_function_name)
+        logger.info("creating a new step function... %s" % step_function_name)
         if setenv:
             os.environ['TIBANNA_DEFAULT_STEP_FUNCTION_NAME'] = step_function_name
             with open(os.getenv('HOME') + "/.bashrc", "a") as outfile:  # 'a' stands for "append"
                 outfile.write("\nexport TIBANNA_DEFAULT_STEP_FUNCTION_NAME=%s\n" % step_function_name)
-        print("deploying lambdas...")
+        logger.info("deploying lambdas...")
         self.deploy_core('all', suffix=suffix, usergroup=usergroup)
         dd_utils.create_dynamo_table(DYNAMODB_TABLE, DYNAMODB_KEYNAME)
         return step_function_name
@@ -885,7 +908,7 @@ class API(object):
                             aws_acc=AWS_ACCOUNT_NUMBER,
                             usergroup=None):
         if not aws_acc or not region_name:
-            print("Please set and export environment variable AWS_ACCOUNT_NUMBER and AWS_REGION!")
+            logger.info("Please set and export environment variable AWS_ACCOUNT_NUMBER and AWS_REGION!")
             exit(1)
         # create a step function definition object
         sfndef = self.StepFunction(dev_suffix, region_name, aws_acc, usergroup)
@@ -903,11 +926,11 @@ class API(object):
                 # get ARN from the error and format as necessary
                 exc_str = str(e)
                 if 'State Machine Already Exists:' not in exc_str:
-                    print('Cannot delete state machine. Exiting...' % exc_str)
+                    logger.error('Cannot delete state machine. Exiting...' % exc_str)
                     raise(e)
                 sfn_arn = exc_str.split('State Machine Already Exists:')[-1].strip().strip("''")
-                print('Step function with name %s already exists!' % sfndef.sfn_name)
-                print('Updating the state machine...')
+                logger.info('Step function with name %s already exists!' % sfndef.sfn_name)
+                logger.info('Updating the state machine...')
                 try:
                     sfn.update_state_machine(
                         stateMachineArn=sfn_arn,
@@ -915,7 +938,7 @@ class API(object):
                         roleArn=sfndef.sfn_role_arn
                     )
                 except Exception as e:
-                    print('Error updating state machine %s' % str(e))
+                    logger.error('Error updating state machine %s' % str(e))
                     raise(e)
             except Exception as e:
                 raise(e)
@@ -936,8 +959,11 @@ class API(object):
         postrunjsonstr = self.log(job_id=job_id, sfn=sfn, postrunjson=True, quiet=True)
         if postrunjsonstr:
             postrunjson = AwsemPostRunJson(**json.loads(postrunjsonstr))
-            job_complete = True
             job = postrunjson.Job
+            if hasattr(job, 'end_time_as_str') and job.end_time_as_str:
+                job_complete = True
+            else:
+                job_complete = False
             log_bucket = postrunjson.config.log_bucket
             instance_type = postrunjson.config.instance_type or 'unknown'
         else:
@@ -955,8 +981,8 @@ class API(object):
         if self.check_metrics_plot(job_id, log_bucket) and \
            self.check_metrics_lock(job_id, log_bucket) and \
            not force_upload:
-            printlog("Metrics plot is already on S3 bucket.")
-            printlog('metrics url= ' + METRICS_URL(log_bucket, job_id))
+            logger.info("Metrics plot is already on S3 bucket.")
+            logger.info('metrics url= ' + METRICS_URL(log_bucket, job_id))
             # open metrics html in browser
             if open_browser:
                 webbrowser.open(METRICS_URL(log_bucket, job_id))
@@ -1010,7 +1036,8 @@ class API(object):
         else:
             try:
                 M = self.TibannaResource(instance_id, filesystem, starttime, endtime)
-                M.plot_metrics(instance_type, directory)
+                top_content = self.log(job_id=job_id, top=True)
+                M.plot_metrics(instance_type, directory, top_content=top_content)
             except Exception as e:
                 raise MetricRetrievalException(e)
             # upload files
@@ -1018,7 +1045,7 @@ class API(object):
             # clean up uploaded files
             for f in M.list_files:
                 os.remove(f)
-        printlog('metrics url= ' + METRICS_URL(log_bucket, job_id))
+        logger.info('metrics url= ' + METRICS_URL(log_bucket, job_id))
         # open metrics html in browser
         if open_browser:
             webbrowser.open(METRICS_URL(log_bucket, job_id))
@@ -1059,7 +1086,7 @@ class API(object):
                 upload('metrics_report.tsv', log_bucket, job_id + '.metrics/')
                 os.remove('metrics_report.tsv')
             else:
-                printlog("cost already in the tsv file. not updating")
+                logger.info("cost already in the tsv file. not updating")
         return cost
 
     def does_dynamo_table_exist(self, tablename):
@@ -1079,7 +1106,7 @@ class API(object):
 
     def create_dynamo_table(self, tablename, keyname):
         if self.does_dynamo_table_exist(tablename):
-            print("dynamodb table %s already exists. skip creating db" % tablename)
+            logger.info("dynamodb table %s already exists. skip creating db" % tablename)
         else:
             response = boto3.client('dynamodb').create_table(
                 TableName=tablename,
@@ -1120,8 +1147,8 @@ class API(object):
         def handle_error(errmsg):
             if ignore_errors:
                 if verbose:
-                    printlog(errmsg)
-                    printlog("continue to remove the other components")
+                    logger.warning(errmsg)
+                    logger.info("continue to remove the other components")
             else:
                 raise Exception(errmsg)
 
@@ -1134,7 +1161,7 @@ class API(object):
         # delete step function
         sfn = 'tibanna_' + self.sfn_type + lambda_suffix
         if verbose:
-            printlog("deleting step function %s" % sfn)
+            logger.info("deleting step function %s" % sfn)
         try:
             boto3.client('stepfunctions').delete_state_machine(stateMachineArn=STEP_FUNCTION_ARN(sfn))
         except Exception as e:
@@ -1143,7 +1170,7 @@ class API(object):
         lambda_client = boto3.client('lambda')
         for lmb in self.lambda_names:
             if verbose:
-                printlog("deleting lambda functions %s" % lmb + lambda_suffix)
+                logger.info("deleting lambda functions %s" % lmb + lambda_suffix)
             try:
                 lambda_client.delete_function(FunctionName=lmb + lambda_suffix)
             except Exception as e:
@@ -1151,12 +1178,12 @@ class API(object):
         # delete IAM policies, roles and groups
         if not do_not_remove_iam_group:
             if verbose:
-                printlog("deleting IAM permissions %s" % sfn)
+                logger.info("deleting IAM permissions %s" % sfn)
             iam = self.IAM(user_group_name)
             iam.delete_tibanna_iam(verbose=verbose, ignore_errors=ignore_errors)
         if purge_history:
             if verbose:
-                printlog("deleting all job files and history")
+                logger.info("deleting all job files and history")
             item_list = dd_utils.get_items(DYNAMODB_TABLE, DYNAMODB_KEYNAME, 'Step Function', sfn, ['Log Bucket'])
             for item in item_list:
                 jobid = item[DYNAMODB_KEYNAME]
@@ -1166,14 +1193,32 @@ class API(object):
                     except Exception as e:
                         if 'NoSuchBucket' in str(e):
                             if verbose:
-                                printlog("log bucket %s missing... skip job %s" % (item['Log Bucket'], jobid))
+                                logger.info("log bucket %s missing... skip job %s" % (item['Log Bucket'], jobid))
                             continue
                     if verbose:
-                        printlog("deleting %d job files for job %s" % (len(keylist), jobid))
+                        logger.info("deleting %d job files for job %s" % (len(keylist), jobid))
                     delete_keys(keylist, item['Log Bucket'])
                 else:
                     if verbose:
-                        printlog("log bucket info missing.. skip job %s" % jobid)
+                        logger.info("log bucket info missing.. skip job %s" % jobid)
             dd_utils.delete_items(DYNAMODB_TABLE, DYNAMODB_KEYNAME, item_list, verbose=verbose)
         if verbose:
-            printlog("Finished cleaning")
+            logger.info("Finished cleaning")
+
+    def create_ami(self, build_from_scratch=True, source_image_to_copy_from=None, source_image_region=None,
+                   ubuntu_base_image=None, make_public=False):
+        args = dict()
+        if build_from_scratch:
+            # build from ubuntu 20.04 image and user data
+            if ubuntu_base_image:
+                args.update({'base_ami': ubuntu_base_image})
+        else:
+            # copy an existing image
+            args.update({'userdata_file': ''})
+            if source_image_to_copy_from:
+                args.update({'base_ami': source_image_to_copy_from})
+            if source_image_region:
+                args.update({'base_region': source_image_region})
+
+        return AMI(**args).create_ami_for_tibanna(make_public=make_public)
+
